/*
 * Tests.cpp
 *
 *  Created on: Feb 8, 2019
 *      Author: andreas
 */

#include <gmock/gmock.h>
#include <gtest/gtest.h>
#include "MockedInterface.h"
#include "ClassToBeTested.h"

using namespace ::testing;

TEST(ExampleTest, simpleExample){
	MockedInterface m;
	EXPECT_CALL(m, askForInfo())
	.WillOnce(Return(InfoContainer("Hello", 42)));

	EXPECT_CALL(m, saveInfo(InfoContainer("HelloHello", 84)))
	.Times(Exactly(1)) 		// Obsolete, because implicit contained in "will once"
	.WillOnce(Return(true));

	ClassToBeTested c(m);
	c.askForInfoAndStoreDuplicated();
}

TEST(ExampleTest, notMatchingExample){
	MockedInterface m;
	EXPECT_CALL(m, askForInfo())
	.WillOnce(Return(InfoContainer("Hello", 42)));

	EXPECT_CALL(m, saveInfo(InfoContainer("Hello", 84)))
	.WillOnce(Return(true));

	ClassToBeTested c(m);
	c.askForInfoAndStoreDuplicated();
}

TEST(ExampleTest, comparePropertyAndFieldExample){
	MockedInterface m;
	EXPECT_CALL(m, askForInfo())
	.WillOnce(Return(InfoContainer("Hello", 42)));

	EXPECT_CALL(m, saveInfo(
			AllOf(Property(&InfoContainer::getString, StartsWith("Hello")),
					Field(&InfoContainer::i, Lt(100)))
	))
	.WillOnce(Return(true));

	ClassToBeTested c(m);
	c.askForInfoAndStoreDuplicated();
}

int main(int argc, char **argv) {
	::testing::InitGoogleTest(&argc, argv);
	return RUN_ALL_TESTS();
}
